#==============================================================================
# package      DE_EPAGES::AmazonPayments::UI::Basket
# base         DE_EPAGES::Order::UI::Basket
# state        private
#==============================================================================
package DE_EPAGES::AmazonPayments::UI::Basket;
use base DE_EPAGES::Order::UI::Basket;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Order::API::LineItemContainer qw ( SavePayment );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Order::UI::OrderProcess;
use JSON;

#==============================================================================
# "Pay with Amazon" button clicked; select Amazon Payments as the payment for this order
sub PayWithAmazonPayments {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $Basket = $Servlet->object;
    my $Container = $Basket->container // return;

    my $Payment = $Container->get('Payment');
    if($Payment && $Payment->instanceOf('LineItemPaymentAmazonPayments')) {
        $Payment->set({ 'TransID' => $Form->value('AmazonRefId') }); # Ensure we are not losing AmazonOrderReferenceId if we remove all products from the basket
        return;
    }

    my $PaymentMethod;
    foreach (@{$Container->get('PaymentMethods')}) {
        if ($_->instanceOf('PaymentMethodAmazonPayments')) {
            $PaymentMethod = $_;
            last;
        }
    }

    eval {
        if (defined $PaymentMethod && isValid($Form->value('AmazonRefId'))) {
            SavePayment($Container, $PaymentMethod);
            $Container->get('Payment')->set({ 'TransID' => $Form->value('AmazonRefId') });
            $Container->recalculate;
        } else {
            Error('AmazonPaymentsError');
        }
    };
    if ( ExistsError() ) {
        Error('INVALID_FORM', {
            'ViewVars'=> { 'FormError' => 0, 'ViewObjectID' => $Basket->id, 'AmazonPaymentsError' => 1 },
            'ViewAction' => 'ViewMultiCheckoutBasket'
        });
    }
    return;
}

#==============================================================================
sub TriggerAmazonPaymentsError {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    my $ErrorCode = $Servlet->params('AddressError')
        // $Servlet->params('WalletError')
        // $Servlet->params('ButtonError');

    LogPayment('AmazonPayments', 'Error calling Amazon Payments', {
        'ErrorCode' => $ErrorCode
    });

    my $Shop = $Basket->get('Shop');
    if ($Shop->get('IsUnityRegistered')) {
        $Servlet->redirect($Basket->get('WebUrlUnitySSL'));
    }

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $Basket->id,
            'AmazonError'  => $ErrorCode
        },
        'ViewAction' => 'ViewMultiCheckoutBasket',
    });
}

1;
