#==============================================================================
# package      DE_EPAGES::AmazonPayments::UI::OrderProcess
# state        public
# description  UI class for Basket Order Process
#==============================================================================
package DE_EPAGES::AmazonPayments::UI::OrderProcess;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use base DE_EPAGES::Order::UI::OrderProcess;
use DE_EPAGES::Order::UI::Basket_Ext;

use strict;

sub ViewMultiCheckoutAmazonAddress {
    my $self = shift;
    my ($Servlet) = @_;

    $self->_registerSession($Servlet);

    # enable execution of template logic even though our 
    # ViewAction is not listed in Order/Templates/SF/SF-MultiCheckoutStep-Content.html
    $Servlet->vars('UseBaseBasketForm', 1);
    $self->SUPER::ViewMultiCheckoutAddress($Servlet);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});
    return;
}

sub ViewMultiCheckoutAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    # enable execution of template logic even though our
    # ViewAction is not listed in Order/Templates/SF/SF-MultiCheckoutStep-Content.html
    $Servlet->vars('UseBaseBasketForm', 1);
    $self->SUPER::ViewMultiCheckoutPayment($Servlet);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});
    return;
}

sub SaveMultiCheckoutAmazonAddress {
    my $self = shift;
    my ($Servlet) = @_;

    TriggerHook('UI_SaveMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});
    $self->SUPER::Save($Servlet);
    TriggerHook('UI_AfterSaveMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});

    $self->CreateOrder($Servlet,'AmazonAddress');
    return;
}

sub SaveMultiCheckoutAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    # do amazon validations.
    TriggerHook('UI_SaveMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});
    $self->SUPER::Save($Servlet);
    TriggerHook('UI_AfterSaveMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});

    # hide the address display on the confirmation page, we replace them with
    # Amazon widgets in read-only mode
    $Servlet->vars('HideConfirmationAddress', 1);
    $self->CreateOrder($Servlet,'AmazonPayment');

    return;
}

sub _registerSession {
    my $self = shift;
    my ($Servlet) = @_;

    DE_EPAGES::Order::UI::Basket_Ext->RegisterSession($Servlet);

    return;
}

1;
