#==============================================================================
# package      DE_EPAGES::Content::API::Constants
# state        private
#------------------------------------------------------------------------------
# description  Some constants used for CSV export/import
#==============================================================================
package DE_EPAGES::Content::API::Constants;
use base Exporter;

use Readonly;
use strict;
our @EXPORT = qw(
    $CSV_SPECIAL_CONTENTITEMS
    @CSV_WARNING_CODES_CONTENT
    CATEGORY_IMAGES_SIZES
    CATEGORY_IMAGE_SIZE_ABBREVIATION
    $SHORTURL_REGEX
    MAX_NESTING_OF_CATEGORIES
    @UNITY_CONTENT_TYPE_ATTRIBUTES_IGNORE
);
use vars qw ( $CSV_SPECIAL_CONTENTITEMS
              @CSV_WARNING_CODES_CONTENT
            );

# attributes that are excluded from csv export/import
# 0 - excluded attrubtes
# 1 - included attributes (even though they are not marked as exportable)
$CSV_SPECIAL_CONTENTITEMS = {
    0 => [ qw( GUID Inherit CreationDate Site ) ],
    1 => [ qw( Class Parent ) ],
};

# possible warnings during CSV import
@CSV_WARNING_CODES_CONTENT = qw(    NO_CONTENTITEMS
                                    VOID_FIELD
                                    CONTENT_EMPTY_ALIAS
                                    CONTENT_EMPTY_CLASSALIAS
                                    NO_SUCH_CONTENTCLASS
                                    NO_PARENT
                                    WRONG_CONTENTCLASS
                                    NO_MAND_VALUE
                                    FeatureLimitExceededSubcategories
                                    FeatureLimitExceeded
                                    SHORTURL_ALREADY_EXISTS
                                    SHORTURL_INVALID_CHARACTER
                                    SHORTURL_NOT_ALLOWED
                                    REFERENCED_IMAGE_MISSING
                                    WRONG_TEMPLATE
                                 );

# this attributes are not exported or imported for producttypes
Readonly our @UNITY_CONTENT_TYPE_ATTRIBUTES_IGNORE => qw(
    VisibleInNavigation
    Title
    URI
    NavigationCaption
    Description
    Text
    Keywords
    ImageSmall
    ImageMedium
    Date
    Text1
    Text2
    Text3
    Image1
    Image2
    Attachment
    AttachmentTitle
    URL
    IsNewWindow
    StartPage/Content-Order
    StartPage/Content-Base
    StartPage/Content-Pages
    StartPage/Content-Products
    Category/Content-Order
    Category/Content-Base
    Category/Content-Pages
    Category/Content-Products
    BusinessHours/Content
);

#========================================================================================
# global   CATEGORY_IMAGES_SIZES
#           | image dimensions for automatic resizing. Hash key is self explaining
#           | Hash value is the image size as array reference [width, height].
#           | ref.hash.array.int
# example  my ($dx, $dy) = @{ CATEGORY_IMAGES_SIZES->{'ImageSmall'} }
#========================================================================================
sub CATEGORY_IMAGES_SIZES { return {
    'ImageSmall'        => [  50,   50],
    'ImageMedium'       => [ 500,  500],
    'Image1'            => [ 200,  200],
    'Image2'            => [ 200,  200],
}}

#========================================================================================
# global   CATEGORY_IMAGE_SIZE_ABBREVIATION
#           | maps image size (see CATEGORY_IMAGES_SIZES)
#             to category image size abbreviations, for example 's' => 'ImageSmall'.
#           | ref.hash.string
# example  my $ImageSmall = CATEGORY_IMAGE_SIZE_ABBREVIATION->{'ImageSmall'} }
#========================================================================================
use constant CATEGORY_IMAGE_SIZE_ABBREVIATION => {
    'ImageSmall'        => 's',
    'ImageMedium'       => 'm',
    'Image1'            => '1',
    'Image2'            => '2'
};

#========================================================================================
# global   MAX_NESTING_OF_CATEGORIES
#           | maximum level for nesting of categories, only used for Unity shops
#========================================================================================
use constant MAX_NESTING_OF_CATEGORIES => 4;

Readonly::Scalar our $SHORTURL_REGEX =>
    qr/
        ^
        [A-Za-z0-9]       # first character can be a letter or digit
        [A-Za-z0-9_\/-]*  # followed by letters, digits, underscore, minus or slash
        (?:\.(?:htm|html))? # optionally followed by a valid extension
        $
    /x;

1;

