use strict;
use warnings;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );

sub Main {

    local $| = 1;

    my $Help;
    my $Company;
    my $Cartridge;

    GetOptions(
               'help'             => \$Help,
               'company=s'        => \$Company,
               'cartridge=s'      => \$Cartridge,
  );

    Usage() if $Help;
    Usage() if !$Company;
    Usage() if !$Cartridge;

    my $CartridgeDir = sprintf ('%s/%s/%s', $ENV{'EPAGES_CARTRIDGES'}, $Company, $Cartridge);
    if(-d $CartridgeDir){
        chdir($CartridgeDir);
        WritePropertiesFile([
            "sonar.projectKey=${Company}::$Cartridge",
            'sonar.clover.reportPath=sonar_coverage.xml',
            'sonar.perl.testHarness.archivePath=sonar_test_report.tgz',
            'sonar.perlcritic.reportPath=sonar_perlcritic_report.txt',
            'sonar.sources='. GetExistingDirectoriesAsCSV(['API', 'DAL', 'UI', 'Scripts', 'Hooks']),
            'sonar.tests='. GetExistingDirectoriesAsCSV(['t']),
        ]);
    }else {
        print"'$CartridgeDir' doesn't exist!\n";
        exit 2;
    }
}

sub WritePropertiesFile {
    my ($aProperties) = @_;
    open( my $fh, '>', 'sonar-project.properties' ) or die "Could not open file 'sonar-project.properties' $!";
    foreach my $Property (@{$aProperties}){
        print $fh $Property."\n";
    }
    close $fh or die "error closing file $!";
}

sub GetExistingDirectoriesAsCSV {
    my ($aDirectories) = @_;
    return join( ',' , grep ( { -d $_ } @{$aDirectories} ) );
}

sub Usage {
    print <<END_USAGE;
Description:
    Creates sonar property file

flags:
    -help          show the command line options
    -cartridge     name of cartridge in company folder
    -company       name of company folder

Example:
    perl $0 -cartridge 4B -company DE_EPAGES
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
