﻿use utf8;
use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::GoogleAdWords::API::GoogleAdWords qw ( FormatAdwordsTrackingCode );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObject );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName = undef;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'nooutput'      => \$NoOutput,
    );

    PrintUsage() if $Help;

    unless( defined $StoreName ) {
        print STDERR "No store specified.\n\n";
        PrintUsage();
    }
    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            my $System = LoadRootObject();
            my $ShopsFolder = $System->child('Shops');
            my $aShopIDs = $ShopsFolder->findChildren();
            my ($Shop, $trackingCodeString, $formattedTrackingCodeString);

            foreach my $ShopID ( @{$aShopIDs} ) {
                eval {
                    $Shop = LoadObject($ShopID);

                    next unless $Shop->class->existsAttribute('GoogleAdWordsHTMLCode');
                    $trackingCodeString = $Shop->get('GoogleAdWordsHTMLCode');

                    next unless isValid($trackingCodeString);
                    $formattedTrackingCodeString = FormatAdwordsTrackingCode($trackingCodeString);

                    $Shop->set({'GoogleAdWordsHTMLCode' => $formattedTrackingCodeString});
                    print 'Resaved tracking code for ' . $Shop->alias . "\n" unless $NoOutput;
                };
                if( ExistsError() ) {
                    my $Error = GetError();
                    print 'Shop: ' .$Shop->alias.' -> '.$Error->message ."\n";
                }
            }

            SynchronizeCache();
        },
    );
    my $time2 = time;
    my $seconds = $time2-$time1;
    printf ("\nneed %.3f seconds\n", $seconds) unless ($NoOutput);
}

sub PrintUsage {
    print <<END_USAGE;
Script to resave the Google AdWords conversion tracking codes.

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                ()
    -storename  name of store                         ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
