﻿package DE_EPAGES::GoogleAdwords::t::GoogleAdwords;
use base DE_EPAGES::Core::API::Test;

use strict;
use utf8;

use DE_EPAGES::GoogleAdWords::API::GoogleAdWords qw (FormatAdwordsTrackingCode);
use Test::More;

sub test {
    my $self = shift;

    $self->test_FormatAdwordsTrackingCode();
    $self->test_noConversionValue_FormatAdwordsTrackingCode();

    done_testing();
    return;
}

sub test_FormatAdwordsTrackingCode {
    my $self = shift;

    my $originalString =
'<!-- Google Code for vm-tgiessler-01 Conversion Page -->
<script type="text/javascript">
/* <![CDATA[ */
var google_conversion_id = 874423081;
var google_conversion_language = "en";
var google_conversion_format = "3";
var google_conversion_color = "ffffff";
var google_conversion_label = "zlQoCMXrjm4Qqcb6oAM";
var google_conversion_value = 1.00;
var google_conversion_currency = "EUR";
var google_remarketing_only = false;
/* ]]> */
</script>
<script type="text/javascript" src="//www.googleadservices.com/pagead/conversion.js">
</script>
<noscript>
<div style="display:inline;">
<img height="1" width="1" style="border-style:none;" alt="" src="//www.googleadservices.com/pagead/conversion/874423081/?value=1.00&amp;currency_code=EUR&amp;label=zlQoCMXrjm4Qqcb6oAM&amp;guid=ON&amp;script=0"/>
</div>
</noscript>';

   my $expectedString =
'<!-- Google Code for vm-tgiessler-01 Conversion Page -->
<script type="text/javascript">
/* <![CDATA[ */
var google_conversion_id = 874423081;
var google_conversion_language = "en";
var google_conversion_format = "3";
var google_conversion_color = "ffffff";
var google_conversion_label = "zlQoCMXrjm4Qqcb6oAM";
var google_conversion_value = "#OrderAmount";
var google_conversion_currency = "EUR";
var google_remarketing_only = false;
/* ]]> */
</script>
<script type="text/javascript" src="//www.googleadservices.com/pagead/conversion.js">
</script>
<noscript>
<div style="display:inline;">
<img height="1" width="1" style="border-style:none;" alt="" src="//www.googleadservices.com/pagead/conversion/874423081/?value=#OrderAmount&amp;currency_code=EUR&amp;label=zlQoCMXrjm4Qqcb6oAM&amp;guid=ON&amp;script=0"/>
</div>
</noscript>';

    my $result = FormatAdwordsTrackingCode($originalString);
    my $idempotentResult = FormatAdwordsTrackingCode($result);

    is($result, $expectedString, 'FormatAdwordsTrackingCode');
    is($idempotentResult, $expectedString, 'FormatAdwordsTrackingCode idempotent');

    return;
}

sub test_noConversionValue_FormatAdwordsTrackingCode {
    my $self = shift;

    my $originalString =
'<!-- Google Code for vm-tgiessler-01 Conversion Page -->
<script type="text/javascript">
/* <![CDATA[ */
var google_conversion_id = 874423081;
var google_conversion_language = "en";
var google_conversion_format = "3";
var google_conversion_color = "ffffff";
var google_conversion_label = "zlQoCMXrjm4Qqcb6oAM";
var google_remarketing_only = false;
/* ]]> */
</script>
<script type="text/javascript" src="//www.googleadservices.com/pagead/conversion.js">
</script>
<noscript>
<div style="display:inline;">
<img height="1" width="1" style="border-style:none;" alt="" src="//www.googleadservices.com/pagead/conversion/874423081/?label=zlQoCMXrjm4Qqcb6oAM&amp;guid=ON&amp;script=0"/>
</div>
</noscript>';

   my $expectedString =
'<!-- Google Code for vm-tgiessler-01 Conversion Page -->
<script type="text/javascript">
/* <![CDATA[ */
var google_conversion_id = 874423081;
var google_conversion_language = "en";
var google_conversion_format = "3";
var google_conversion_color = "ffffff";
var google_conversion_label = "zlQoCMXrjm4Qqcb6oAM";
var google_remarketing_only = false;
/* ]]> */
</script>
<script type="text/javascript" src="//www.googleadservices.com/pagead/conversion.js">
</script>
<noscript>
<div style="display:inline;">
<img height="1" width="1" style="border-style:none;" alt="" src="//www.googleadservices.com/pagead/conversion/874423081/?label=zlQoCMXrjm4Qqcb6oAM&amp;guid=ON&amp;script=0"/>
</div>
</noscript>';

    my $result = FormatAdwordsTrackingCode($originalString);
    my $idempotentResult = FormatAdwordsTrackingCode($result);

    is($result, $expectedString, 'FormatAdwordsTrackingCode noConversionValue');
    is($idempotentResult, $expectedString, 'FormatAdwordsTrackingCode noConversionValue idempotent');

    return;
}

__PACKAGE__->runScript();
1;
