#========================================================================================
# package      DE_EPAGES::IGFS::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::IGFS::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::IGFS::API::Payment qw ( FetchTransactionStatus UpdateTransactionStatus );
use DE_EPAGES::IGFS::API::TransStatus qw (
  STATUS_AUTHORISED
  STATUS_CANCELLED
  STATUS_FAILED
);
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByGUID LoadObjectByGUID );
use DE_EPAGES::Order::UI::Basket;

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return GoogleCheckout as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {'IGFS'}

#==============================================================================
# This callback is sent to the success link.
#==============================================================================
sub PaymentBacklinkIGFS {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);
    my $hFormValues     = $Servlet->params();
    my $Shop            = $Servlet->object();
    my $PaymentLineItemGUID = $hFormValues->{'GUID'};
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);
    my $Form            = $Servlet->form;
    my $ErrorViewAction = 'ViewPaymentIGFSResult';

    # check if GUID exists.
    unless ( isValid($PaymentLineItemGUID) && ExistsObjectByGUID($PaymentLineItemGUID)) {
        if ($Shop->isUnity()) {
            return;
        } else {
            $Form->executeFormError( {
                    'Name'       => 'GUID',
                    'Reason'     => 'BadCallback_NoPaymentGUID',
                    'ViewAction' => $ErrorViewAction,
            } );
        }
    }

    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);
    my $BasketOrder     = $PaymentLineItem->container->parent;
    my $hValues         = _ProcessPaymentParameters( $Servlet, $PaymentLineItem );
    if ( $hValues->{'TransStatus'} eq STATUS_CANCELLED ) {
        if ( $Shop->isUnity()) {
            $PaymentLineItem->set({'PaymentErrorCode' => 'Canceled'});
            $Servlet->redirect($BasketOrder->get('WebUrlUnitySSL'));
            return;
        } else {
            $Form->executeFormError( {
                    'Name'        => 'GUID',
                    'Reason'      => 'PaymentProcess_Cancelled',
                    'ViewAction'  => $ErrorViewAction,
                    'ObjectID'    => $BasketOrder->id,
                    'ObjectClass' => $BasketOrder->class->alias,
            } );
        }
    }

    if ( $hValues->{'TransStatus'} eq STATUS_FAILED ) {
        if ($Shop->isUnity()) {
            $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'});
            $Servlet->redirect($BasketOrder->get('WebUrlUnitySSL'));
            return;
        } else {
            $Form->executeFormError( {
                    'Name'        => 'GUID',
                    'Reason'      => 'PaymentProcess_Failed',
                    'ViewAction'  => $ErrorViewAction,
                    'ObjectID'    => $BasketOrder->id,
                    'ObjectClass' => $BasketOrder->class->alias,
            } );
        }
    }

    if ( defined $hValues->{'ViewObject'} ) {
        my $CustomerOrder = $hValues->{'ViewObject'};
        $Servlet->vars( 'ViewObjectID', $CustomerOrder->id );
        $Servlet->registerSession( $CustomerOrder->get('User'), 0 );
    }
    return;
}

#==============================================================================
sub _ProcessPaymentParameters {
    my ( $Servlet, $PaymentLineItem ) = @_;

    my $Shop            = $Servlet->object;
    my $Form            = $Servlet->form;
    my $BasketOrder     = $PaymentLineItem->container->parent;
    my $ErrorViewAction = 'ViewPaymentIGFSResult';

    # check if GUID belongs to this shop.
    if ( $PaymentLineItem->getSite()->id != $Shop->id ) {
        if ($Shop->isUnity()) {
            return;
        } else {
            $Form->executeFormError( {
                    'Name'       => 'GUID',
                    'Reason'     => 'BadCallback_ShopIDMisMatch',
                    'ViewAction' => $ErrorViewAction
            } );
        }
    }
    my $returnStatus = FetchTransactionStatus( $Servlet, $PaymentLineItem );
    my $hValues = UpdateTransactionStatus( $PaymentLineItem, $returnStatus );

    # case 1: successful return to basket.
    if (    $BasketOrder->instanceOf('Basket')
        and $hValues->{'TransStatus'} eq STATUS_AUTHORISED ) {

        # call basket2order
        $BasketOrder = DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $BasketOrder );

        # case 2: order exists and either failed return status
    }
    elsif ( $BasketOrder->instanceOf('CustomerOrder') and $hValues->{'TransStatus'} ne STATUS_AUTHORISED ) {

        # set order cancelled
        $BasketOrder->set( { 'CancelledOn' => $hValues->{'TransTime'} } );
    }

    # let's set paid on successful payment status
    if ( $hValues->{'TransStatus'} eq STATUS_AUTHORISED ) {

        # set order paid
        $BasketOrder->set( { 'PaidOn' => $hValues->{'TransTime'} } )
          unless ( defined $BasketOrder->get('PaidOn') );
        $hValues->{'ViewObject'} = $BasketOrder;
    }
    return $hValues;
}

1;
