var should = require('should');
var testrunner = require('./testrunner');
describe('qUnit test runner', function() {
    describe('#areParamsValid()', function() {
        it('should return true if all mandatory params are provided', function() {
            var validParams = ['-v', '6.17.58', '-storetype', 'Store', '-vm', 'vm-mmustermann.intern.epages.de'];
            testrunner.areParamsValid(validParams).should.be.ok();
        });
        it('should return true if an optional param is provided', function() {
            var optionalParams = ['-v', '6.17.58', '-storetype', 'Store', '-vm', 'vm-mmustermann.intern.epages.de', '-c', 'Order'];
            testrunner.areParamsValid(optionalParams).should.be.ok();
        });
        it('should return false if a mandatory param name is missing', function() {
            var invalidParams = ['6.17.58', '-storetype', 'Store', '-vm', 'vm-mmustermann.intern.epages.de'];
            testrunner.areParamsValid(invalidParams).should.not.be.ok();
        });
        it('should return false if a mandatory param value is missing', function() {
            var invalidParams = ['-v', '-storetype', 'Store', '-vm', 'vm-mmustermann.intern.epages.de'];
            testrunner.areParamsValid(invalidParams).should.not.be.ok();
        });
        it('should return false if an invalid param is provided', function() {
            var invalidParams = ['-v', '-storetype', 'Store', '-vm', 'mmustermann@epages.de'];
            testrunner.areParamsValid(invalidParams).should.not.be.ok();
        });
        it('shoud return false if no params are provided', function() {
            testrunner.areParamsValid([]).should.not.be.ok();
        });
        it('should return false if -c is a parameter but no Cartridge was provided', function() {
            var invalidParams = ['-v', '6.17.58', '-storetype', 'Store', '-c', '-vm', 'vm-mmustermann.intern.epages.de'];
            testrunner.areParamsValid(invalidParams).should.not.be.ok();
        });
    });
    describe('#getPathToTest()', function() {
        it('should return the path to the lib folder of the given cartridge if -c was provided as parameter', function() {
            var validParams = ['-c', 'Order'];
            testrunner.getPathToTest(validParams).should.be.equal('../../../Order/Data/Public/lib');
        });
        it('should provide the current cartridge folder if no -c was provided', function() {
            var validParams = ['C:\\Program Files\\nodejs\\node.exe', 'D:\\VMSharedFolders\\vm1\\Cartridges\\DE_EPAGES\\JQuery\\t\\qUnit\\index.js'];
            testrunner.getPathToTest(validParams).should.be.equal('../../../JQuery/Data/Public/lib');
        });
        it('should provide the main Catridge folder if -all was provided', function() {
            var validParams = ['-all'];
            testrunner.getPathToTest(validParams).should.be.equal('../../..');
        });
        it('should still return the main Cartridge folder, if -all is provided but also -c is provided', function() {
            var validParams = ['-all', '-c', 'Order'];
            testrunner.getPathToTest(validParams).should.be.equal('../../..');
        });
    });
    describe('#getTestFiles()', function() {
        it('should return an array of files to test', function() {
            var pathToTest = '../../../Order/Data/Public/lib';
            var resultArray = ['../de_epages/order/ui/documentgenerator-test.js'];
            testrunner.getTestFiles([pathToTest]).should.be.eql(resultArray);
        });
    });
});
