var path = require('path'),
    fs = require('fs');
'use strict';

class TestRunner {

    constructor() {}

    areParamsValid(params) {
       if(((params.indexOf('-v') >= 0) && ( /\d{1,2}\.\d{1,2}\.\d{1,2}/.test(params[params.indexOf('-v') + 1])) )
        && ((params.indexOf('-storetype') >= 0) && ( /[A-Z][A-Za-z]+/.test(params[params.indexOf('-storetype') + 1])))
        && ((params.indexOf('-vm') >= 0) && ( /[\w-.]+\.+[a-z]+/.test(params[params.indexOf('-vm') + 1])))) {

            if ((params.indexOf('-c') >= 0) && !(/[A-Z][A-Za-z]+/.test(params[params.indexOf('-c') + 1]))) {
                return false;
            } else {
                return true;
            }
        } else {
            return false;
        }
    }

    getPathToTest(params) {
        var cartridge,
            pathToTest,
            testFolder,
            cartridgePlusSubs;

        if (params.indexOf('-all') >= 0) {
            pathToTest = '../../..';

        } else if ((params.indexOf('-c') >= 0) && !(params.indexOf('-all') >= 0)) {
            cartridge = params[params.indexOf('-c') + 1];
            pathToTest = '../../../' + cartridge + '/Data/Public/lib';

        } else {
            testFolder = params[1];
            cartridgePlusSubs = testFolder.substring(((testFolder.indexOf('EPAGES')) + 7), testFolder.length);

            if ( (cartridgePlusSubs.indexOf('\\')) != -1 ) {
                cartridge = cartridgePlusSubs.substring( 0, cartridgePlusSubs.indexOf('\\'));
            } else {
                cartridge = cartridgePlusSubs;
            }
            pathToTest = '../../../' + cartridge + '/Data/Public/lib';
        }

        return pathToTest;
    }

    getTestFiles(directories) {
        var file,
            stat;

        if (directories.length == 0) {
            return [];
        } else if (directories.length == 1) {
            file = directories[0];
            stat = fs.statSync(file);

            return (stat.isFile() && this._isTestFile(file)) ? [this._correctPath(file)] : stat.isDirectory() ? this.getTestFiles(this._subDirectories(file)) : [];

        } else {

            return this.getTestFiles([directories.shift()]).concat(this.getTestFiles(directories));
        }
    }

    _isTestFile(file) {
        var regex = /\..*?\/(ep|de_epages|ckeditor)\//;

        return (file.substring((file.length - 8), file.length) === '-test.js' && regex.test(file));
    }

    _correctPath(filePath) {
        return '..' + filePath.substring(( filePath.indexOf('lib') + 3 ), ( filePath.length ));
    }

    _subDirectories(dir) {
        return fs.readdirSync(dir).map(function(fileOrDir) {return dir + '/' + fileOrDir});
    }

    printInstructions() {
        console.log("------");
        console.log("Usage:");
        console.log("------");
        console.log("index.js -vm [VM-name] -v [version] -storetype [Storename] [optional Options]\n");
        console.log("------------");
        console.log("Description:");
        console.log("------------");
        console.log("Execute all QUnit tests in current (or requested) Cartridge(s).\n");
        console.log("--------------------");
        console.log("Mandatory Arguments:");
        console.log("--------------------");
        console.log("   -storetype      specify the store you want to run the tests for (eg. 'Store')");
        console.log("   -v              specify the version of epages you want to run the tests on (eg. '7.4.0')");
        console.log("   -vm             specify the fqdn of the machine you want to run the tests on (eg. 'ep.intern.epages.de')\n");
        console.log("-------------------");
        console.log("Optional Arguments:");
        console.log("-------------------");
        console.log("   -all            run tests for all cartridges (no argument)");
        console.log("   -c              run all tests in specified cartridge (eg. 'JQuery')\n                   default: current Cartridge");
        console.log("   -debug          show javascript errors during execution of tests (no argument)");
        console.log("   -ssl            use ssl for running tests (no argument)");
        console.log("   -threads        run tests parallel with specified amount of threads (eg. '6')\n                   default: 4");
        console.log("   -timeout        set timeout for execution of each test in seconds (eg. '60')\n                   default: 30\n");
        console.log("------------");
        console.log("Exit status:");
        console.log("------------");
        console.log(" -1    if no test ran,");
        console.log("  0    if any test failed,");
        console.log(" >0    number of successful tests\n");
        console.log("---------");
        console.log("Examples:");
        console.log("---------");
        console.log("node index.js -vm vm-tgiessler-01.intern.epages.de -v 7.0.0 -storetype Store -all ");
        console.log("node index.js -vm vm-tgiessler-01.intern.epages.de -v 7.0.0 -storetype Store -c JQuery -debug -ssl -threads 6 -timeout 60 \n");
    }

}

module.exports = new TestRunner();
