#========================================================================================
# package      DE_EPAGES::Mollie::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for basket or order
#========================================================================================
package DE_EPAGES::Mollie::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Mollie::API::Token;
use DE_EPAGES::Mollie::API::PaymentRequest;
use DE_EPAGES::Mollie::API::MolliePayment;

use strict;
use Readonly;

#========================================================================================
# function     ViewPaymentMollie
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentMollie( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for classical/mobile checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewPaymentMollie {
    my $self    = shift;
    my ($Servlet) = @_;

    my $MolliePayment = DE_EPAGES::Mollie::API::MolliePayment->new();
    my $LineItemPayment = $Servlet->object()->get('LineItemContainer')->get('Payment');
    my $Locale = GetInfoLanguage( $Servlet->languageID )->{'DefaultLocaleID'};
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');
    my $GUID = $LineItemPayment->get('GUID');
    my $isMobile = $Servlet->vars('IsMobile');

    eval {
        $MolliePayment->createPayment($LineItemPayment, $Locale, $isMobile);
    };

    if ( ExistsError() ) {
        my $BasketOrOrder = $Servlet->object();
        my $Shop = $BasketOrOrder->get('Shop');
        if ($Shop->get('IsUnityRegistered')) {
            $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
            $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));
            return;
        }

        my $Error = GetError();
        LogPayment('Mollie','ViewPaymentMolliePay' , $Error->longMessage(), 'Error');
        my $Form = $Servlet->form;
        $Form->executeFormError( {
                'Name'       => 'CreatePayment',
                'Reason'     => 'CreatePayment_Failed',
                'ViewAction' => 'ViewPaymentMollieError',
            }
        );
    }

    $Servlet->vars('PaymentUrl', $MolliePayment->{'PaymentURL'});
    $self->_updateLineItemPayment($LineItemPayment, $PaymentMethod, $GUID, $MolliePayment->{'TransactionID'});

    return;
}

#==============================================================================
sub _updateLineItemPayment {
    my $self = shift;
    my ($LineItemPayment, $PaymentMethod, $GUID, $TransactionID) = @_;

    $LineItemPayment->set( {
        'TransID' => $GUID,
        'TransTime' => GetCurrentDateTime(),
        'TransID2' => $TransactionID
     });

    return;
}
#========================================================================================
# function     ViewMultiPaymentMollie
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewMultiPaymentMollie( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for multi-step checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewMultiPaymentMollie {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash());

    $self->ViewPaymentMollie($Servlet);

    return;
}

1;