#========================================================================================
# package      DE_EPAGES::Mollie::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  UI Controller for shop
#========================================================================================
package DE_EPAGES::Mollie::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Mollie::API::Constants qw (
    %EPAGES_TRANSACTION_STATUS
    INTERNAL_CARTRIDGE_NAME
);
use DE_EPAGES::Mollie::API::PaymentDetailsRequest;
use DE_EPAGES::Mollie::API::Token;
use DE_EPAGES::Mollie::API::UpdateOrder;
use DE_EPAGES::Mollie::API::UpdateOrderStatus;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::Order::UI::Basket;

use strict;



#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    return INTERNAL_CARTRIDGE_NAME;
}

#========================================================================================
# function     PaymentBacklinkMollie
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentBacklinkMollie( $Servlet );
#----------------------------------------------------------------------------------------
# description  Mollie backlink.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentBacklinkMollie {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);
    my $hFormValues = $Servlet->params();
    LogPayment('Mollie','Backlink parameters', $hFormValues);

    $self->_resetServletVariables($Servlet);

    my $Form = $Servlet->form();
    my $PaymentLineItemGUID = $hFormValues->{'MGUID'};

    my $UpdateOrder = DE_EPAGES::Mollie::API::UpdateOrder->new();
    my $hResponse = $UpdateOrder->update($Servlet,$PaymentLineItemGUID);
    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);
    my $LineItemContainer = $PaymentLineItem->container();
    my $BasketOrOrder = $LineItemContainer->parent();

    my $Order = $hResponse->{'Order'};
    if (defined $Order) {
        $BasketOrOrder = $Order;
    }

    if ( $hResponse->{'isCancelBacklink'} ){
        $self->_showCancelBacklink($Form, $LineItemContainer, $BasketOrOrder, $Servlet);
    }

    $self->_showCustomerOrder($BasketOrOrder, $Servlet);

    return;
}

sub _resetServletVariables {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    return;
}

sub _showCancelBacklink {
    my $self = shift;
    my ($Form, $LineItemContainer, $BasketOrOrder, $Servlet) = @_;

    my $Shop = $BasketOrOrder->get('Shop');
    if ($Shop->get('IsUnityRegistered')) {
        my $LineItemPayment = $LineItemContainer->get('Payment');
        $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
        $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));
        return;
    }

    LogPayment(
        'Mollie',
        'PaymentProcess_NotPaid',
        {
            'Name'        => 'MGUID',
            'Reason'      => 'PaymentProcess_Canceled',
            'ViewAction'  => 'ViewPaymentMollieResult',
            'ObjectID'    => $BasketOrOrder->id(),
            'ObjectClass' => $BasketOrOrder->class()->alias(),
        }
    );
    $Form->executeFormError(
        {
            'Name'        => 'MGUID',
            'Reason'      => 'PaymentProcess_Canceled',
            'ViewAction'  => 'ViewPaymentMollieResult',
            'ObjectID'    => $BasketOrOrder->id(),
            'ObjectClass' => $BasketOrOrder->class()->alias(),
        }
    );
    return;
}

sub _showCustomerOrder {
    my $self = shift;
    my ($CustomerOrder, $Servlet) = @_;

    if (defined  $CustomerOrder) {
        $Servlet->vars( 'ViewObjectID', $CustomerOrder->id );
        $Servlet->registerSession($CustomerOrder->get('User'),0 );
    }

    return;
}


1;
