#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::UI_Basket
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Basket
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::UI_Basket;
use base DE_EPAGES::Core::API::Test;
use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::PayPalExpress::UI::Basket;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

sub test {
    my $self = shift;

    $self->test_InvalidBasket();
    $self->test_ShortcutButton_with_MultiStepCheckout();
    $self->test_ShortcutButton_with_ClassicalCheckout();
    $self->test_ShortcutButton_with_MobileCheckout();
    $self->test_ClassicOrMultistepCheckout_without_ShortcutButton();
    $self->test_ErrorHandling();
    $self->test_ShortcutButton_ErrorRedirection();
    $self->test_ClassicOrMultistepCheckout_ErrorRedirection();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_InvalidBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_with_InvalidBasket();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForMultistepCheckout();

    my $Result = $UIBasket->PayPalExpressReturn( $Servlet );

    is( $Result, undef, "$SubTestName - tests error handling if the importBasket was not fine" );
}
#----------------------------------------------------------------------------------------
sub test_ShortcutButton_with_MultiStepCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_MultistepCheckout_with_ShortcutButton();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForMultistepCheckout();

    my $Result = $UIBasket->PayPalExpressReturn( $Servlet );

    is( $Result, undef, "$SubTestName - tests if ExpressButton call with MultistepCheckout was used" );
    my $hServletVars = $Servlet->vars();
    my $hExpectedServletVars = {'ViewAction' => 'ViewMultiCheckoutConfirmation'};
    is_deeply( $hServletVars, $hExpectedServletVars, "$SubTestName - tests if the multistep view action is set" );
}
#----------------------------------------------------------------------------------------
sub test_ShortcutButton_with_ClassicalCheckout{
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_ClassicCheckout_with_ShortcutButton();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForClassicCheckout();

    my $Result = $UIBasket->PayPalExpressReturn( $Servlet );

    is( $Result, undef, "$SubTestName - tests if ExpressButton call with MultistepCheckout was used" );

    my $hServletVars = $Servlet->vars();
    my $hExpectedServletVars = {'ViewAction' => 'ViewOffer'};
    is_deeply( $hServletVars, $hExpectedServletVars, "$SubTestName - tests if the classic view action is set" );
}
#----------------------------------------------------------------------------------------
sub test_ShortcutButton_with_MobileCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_MultistepCheckout_with_ShortcutButton();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForMobileCheckout();

    my $Result = $UIBasket->PayPalExpressReturn( $Servlet );

    is( $Result,undef,"$SubTestName - tests if ExpressButton call with Mobile was used" );

    my $hServletVars = $Servlet->vars();
    my $hExpectedServletVars = {'ViewAction' => 'ViewOffer'};
    is_deeply( $hServletVars, $hExpectedServletVars, "$SubTestName - tests if the classic view action is set" );
}
#----------------------------------------------------------------------------------------
sub test_ClassicOrMultistepCheckout_without_ShortcutButton {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_ClassicOrMultistepCheckout_without_ShortcutButton();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForClassicCheckout();

    my $Result = $UIBasket->PayPalExpressReturn( $Servlet );

    is( $Result, 'FakeOrder', "$SubTestName - tests if ClassicalCheckout was used" );
}
#----------------------------------------------------------------------------------------
sub test_ErrorHandling{
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService_ErrorTest();
    my $UIBasket = $self->_buildUIBasket( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForClassicCheckout();

    $UIBasket->PayPalExpressReturn( $Servlet );

    my $hServletVars = $Servlet->vars();
    like( $hServletVars->{'ErrorGUID'}, qr/^[0-9A-F\-]+$/i, "$SubTestName - test the vars output for ErrorGUID" );
    is( $hServletVars->{'ViewAction'}, 'ViewPayPalExpressFailed', "$SubTestName - test the vars output for ViewAction" );
    is( $hServletVars->{'ErrorCode'}, 'GetExpressCheckoutDetailsError', "$SubTestName - test the vars output for ErrorCode" );
    is( $hServletVars->{'ErrorMessage'}, 'testNativeDie', "$SubTestName - test the vars output for ErrorMessage" );
}
#----------------------------------------------------------------------------------------
sub test_ShortcutButton_ErrorRedirection {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_ShortcutButton();
    my $UIBasket = $self->_buildUIBasket_ShortcutButton( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForClassicCheckout();

    $UIBasket->PayPalExpressReturn( $Servlet );

    my $hServletVars = $Servlet->vars();
    is( $hServletVars->{'ViewAction'}, 'ViewPayPalExpressFailed', "$SubTestName - test the vars output for ViewAction" );
    is( $hServletVars->{'ErrorCode'}, 'PayPalExpressInvalid', "$SubTestName - test the vars output for ErrorCode" );
    is( $hServletVars->{'PPEErrorCode'}, 'errorCode', "$SubTestName - test the vars output for PPEErrorCode" );
    is( $hServletVars->{'PayPalMessage'}, 'errorMessage', "$SubTestName - test the vars output for PayPalMessage" );
}
#----------------------------------------------------------------------------------------
sub test_ClassicOrMultistepCheckout_ErrorRedirection {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_buildBasketService_ClassicCheckout();
    my $UIBasket = $self->_buildUIBasket_ClassicCheckout( $BasketService );
    my $Servlet = $self->_buildServlet_with_OrderForClassicCheckout();

    $UIBasket->PayPalExpressReturn( $Servlet );

    my $hServletVars = $Servlet->vars();
    is( $hServletVars->{'ViewAction'}, 'ViewPayPalExpressFailed',"$SubTestName - test the vars output for ViewAction" );
    is( $hServletVars->{'ErrorCode'}, 'PayPalExpressInvalid',"$SubTestName - test the vars output for ErrorCode" );
    is( $hServletVars->{'PPEErrorCode'}, 'errorCode',"$SubTestName - test the vars output for PPEErrorCode" );
    is( $hServletVars->{'PayPalMessage'}, 'errorMessage',"$SubTestName - test the vars output for PayPalMessage" );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildBasketService_ShortcutButton {
    my $self = shift;
    my $hSettings = {
        'IsButtonTransaction' => 1,
    };
    return $self->_createBasketService_ShortcutButton( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _buildBasketService_ClassicCheckout {
    my $self = shift;
    my $hSettings = {
        'IsButtonTransaction' => 0,
    };
    return $self->_createBasketService_ShortcutButton( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _createBasketService_ShortcutButton {
    my $self = shift;
    my ( $hSettings ) = @_;
    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'importBasket', 1, [{'object'=>'DE_EPAGES::Order::API::Object::Basket'}] );
    $Mockify->addMockWithReturnValue( 'wasTriggeredWithShortcutButton', $hSettings->{'IsButtonTransaction'} );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildUIBasket_ShortcutButton {
   my $self = shift;
    my ( $BasketService ) = @_;

    my $hSettingsExpressCheckoutDetails = {
        'ReturnValue_doRequestForShortcutButtonCheckout' => 0,
        'ReturnValue_doRequestForClassicOrMultistepCheckout' => 1,
    };

    my $hSettingsUIBasket = {
        'BasketService' => $BasketService,
        'ExpressCheckoutDetails' => $self->_createGetExpressCheckoutDetails( $hSettingsExpressCheckoutDetails ),
    };
    return $self->_createUIBasket( $hSettingsUIBasket );
}
#----------------------------------------------------------------------------------------
sub _buildUIBasket_ClassicCheckout {
   my $self = shift;
    my ( $BasketService ) = @_;

    my $hSettingsExpressCheckoutDetails = {
        'ReturnValue_doRequestForShortcutButtonCheckout' => 0,
        'ReturnValue_doRequestForClassicOrMultistepCheckout' => 0,
    };

    my $hSettingsUIBasket = {
        'BasketService' => $BasketService,
        'ExpressCheckoutDetails' => $self->_createGetExpressCheckoutDetails( $hSettingsExpressCheckoutDetails ),
    };
    return $self->_createUIBasket( $hSettingsUIBasket );
}
#----------------------------------------------------------------------------------------
sub _buildUIBasket {
    my $self = shift;
    my ( $BasketService ) = @_;

    my $hSettingsExpressCheckoutDetails = {
        'ReturnValue_doRequestForShortcutButtonCheckout' => 1,
        'ReturnValue_doRequestForClassicOrMultistepCheckout' => 1,
    };

    my $hSettingsUIBasket = {
        'BasketService' => $BasketService,
        'ExpressCheckoutDetails' => $self->_createGetExpressCheckoutDetails( $hSettingsExpressCheckoutDetails ),
    };
    return $self->_createUIBasket( $hSettingsUIBasket );
}
#----------------------------------------------------------------------------------------
sub _createUIBasket {
    my $self = shift;
    my ( $hSettings ) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::UI::Basket',
                   $aParameterList
    );
    $Mockify->addMockWithReturnValue( '_createBasketService', $hSettings->{'BasketService'} );
    $Mockify->addMockWithReturnValue( '_createWebserviceGetExpressCheckoutDetails', $hSettings->{'ExpressCheckoutDetails'} );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_LogError',
        1,
        [{'object' => 'DE_EPAGES::Core::API::Error'}]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_doBasket2Order',
        'FakeOrder',
        [
            {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
            {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
        ]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_logRequest',
        undef,
        [ {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'} ]
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetExpressCheckoutDetails {
    my $self = shift;
    my ( $hSettings ) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails',
                   $aParameterList
              );
    my $aParameterTypes = [
        {'object' => 'DE_EPAGES::PayPalExpress::API::Webservice::BasketService'},
        {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
    ];
    my $getErrorMessageValue = 'errorMessage';
    my $getErrorCodeValue = 'errorCode';

    $Mockify->addMockWithReturnValueAndParameterCheck(
        'doRequestForShortcutButtonCheckout',
        $hSettings->{'ReturnValue_doRequestForShortcutButtonCheckout'},
        $aParameterTypes
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'doRequestForClassicOrMultistepCheckout',
        $hSettings->{'ReturnValue_doRequestForClassicOrMultistepCheckout'},
        $aParameterTypes
    );
    $Mockify->addMockWithReturnValue( 'getErrorMessage', $getErrorMessageValue );
    $Mockify->addMockWithReturnValue( 'getErrorCode', $getErrorCodeValue );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildBasketService_with_InvalidBasket {
    my $self = shift;
    my $hSettings = {
        'ImportBasket' => 0,
    };
    return $self->_createBasketService( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _buildBasketService_MultistepCheckout_with_ShortcutButton {
    my $self = shift;
    my $hSettings = {
        'ImportBasket' => 1,
        'IsButtonTransaction' => 1,
        'IsMultistepCheckout' => 1,
    };
    return $self->_createBasketService( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _buildBasketService_ClassicCheckout_with_ShortcutButton {
    my $self = shift;
    my $hSettings = {
        'ImportBasket' => 1,
        'IsButtonTransaction' => 1,
        'IsMultistepCheckout' => 0,
    };
    return $self->_createBasketService( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _buildBasketService_ClassicOrMultistepCheckout_without_ShortcutButton {
    my $self = shift;
    my $hSettings = {
        'ImportBasket' => 1,
        'IsButtonTransaction' => 0,
        'IsMultistepCheckout' => 0,
        'Basket' => $self->_createBasket(),
    };
    return $self->_createBasketService( $hSettings );
}
#----------------------------------------------------------------------------------------
sub _createBasketService {
    my $self = shift;
    my ( $hSettings ) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList,
    );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'importBasket', $hSettings->{'ImportBasket'}, [{'object'=>'DE_EPAGES::Order::API::Object::Basket'}] );
    $Mockify->addMockWithReturnValue( 'wasTriggeredWithShortcutButton', $hSettings->{'IsButtonTransaction'} );
    $Mockify->addMockWithReturnValue( 'isMultistepCheckout', $hSettings->{'IsMultistepCheckout'} );
    $Mockify->addMockWithReturnValue( 'getBasket',$self->_createBasket() );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasketService_ErrorTest {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList,
    );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'importBasket', 1, [{'object'=>'DE_EPAGES::Order::API::Object::Basket'}] );
    $Mockify->addMock( 'wasTriggeredWithShortcutButton', sub { Error('testNativeDie'); } );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildServlet_with_OrderForClassicCheckout {
    my $self = shift;
    my $IsMultistepCheckout = 0;
    my $ServletSettings = {
        'BasketObject' => $self->_createBasket( $IsMultistepCheckout ),
        'UserObject' => $self->_createUser(),
        'IsMobileSwitch' => 0,
    };
    return $self->_createServlet( $ServletSettings );
}
#----------------------------------------------------------------------------------------
sub _buildServlet_with_OrderForMultistepCheckout {
    my $self = shift;
    my $IsMultistepCheckout = 1;
    my $ServletSettings = {
        'BasketObject' => $self->_createBasket( $IsMultistepCheckout ),
        'UserObject' => $self->_createUser(),
        'IsMobileSwitch' => 0,
    };
    return $self->_createServlet( $ServletSettings );
}
#----------------------------------------------------------------------------------------
sub _buildServlet_with_OrderForMobileCheckout {
    my $self = shift;
    my $IsMultistepCheckout = 1;
    my $ServletSettings = {
        'BasketObject' => $self->_createBasket( $IsMultistepCheckout ),
        'UserObject' => $self->_createUser(),
        'IsMobileSwitch' => 1,
    };
    return $self->_createServlet( $ServletSettings );
}
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($hServletSettings) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Presentation::Servlet::Permission',
                   $aParameterList
    );
    $Mockify->addMockWithReturnValue( 'object', $hServletSettings->{'BasketObject'} );
    $Mockify->addMockWithReturnValue( 'user', $hServletSettings->{'UserObject'} );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'cookies', undef, [{'string'=>'RedirectPPE'}, {'int'=>0}] );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'params', $hServletSettings->{'IsMobileSwitch'}, [{'string'=>'IsMobile'}] );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($IsMultistepCheckout) = @_;
    my $Shop = $self->_createShop( $IsMultistepCheckout );


    my $hAtributes = {
        'User' => $self->_createUser(),
        'Site' => $Shop,
        'Shop' => $Shop,
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAtributes );

    return $Basket;
}
#----------------------------------------------------------------------------------------
sub _createUser {
    my $self = shift;

    my $hAtributes = {
        'ID' => 12345,
    };
    my $User = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAtributes );

    return $User;
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($IsMultistepCheckout) = @_;
    my $hAtributes = {
        'OrderProcess' => $IsMultistepCheckout,
        'IsUnityRegistered' => 0
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAtributes );

    return $Shop;
}
__PACKAGE__->runScript();