#========================================================================================
# package      DE_EPAGES::DE_EPAGES::PayPalPlus::t::unit::PaymentWallDataBuilder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for PaymentWallDataBuilder
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::PaymentWallDataBuilder;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use strict;

sub test {
    my $self = shift;
    $self->test_extractData_Successful_Sandbox();
    $self->test_extractData_Successful_Live();
    $self->test_extractData_WrongURL();
    $self->test_extractData_MissingIsInTestMode();
    $self->test_extractData_MissingLineItemPayment();
    done_testing();
}

sub test_extractData_Successful_Sandbox {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder->new();

    my $hParams = {'ApprovalURL' => 'http://some.to/url'};
    my $IsInTestMode = 1;
    my $LocaleId = 'de_DE';
    my $CountryCode = 'EN';
    my $LineItemPaymentPayPalPlus = $self->_buildLineItemPaymentPayPalPlus($hParams, $IsInTestMode, $LocaleId, $CountryCode);

    my $wasSuccessful = $PaymentWallDataBuilder->extractData($LineItemPaymentPayPalPlus);
    my $hData = $PaymentWallDataBuilder->getData();

    is($wasSuccessful, 1, "$SubTestName - Test if call was successful");
    is($hData->{'approvalUrl'},'http://some.to/url', "$SubTestName - Test if URL is right");
    is($hData->{'mode'},'sandbox', "$SubTestName - Test if mode is right");
    is($hData->{'language'},'de_DE', "$SubTestName - Test if URL is right");
    is($hData->{'country'},'EN', "$SubTestName - Test if mode is right");
}

sub test_extractData_Successful_Live {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder->new();

    my $hParams = {'ApprovalURL' => 'http://some.to/url'};
    my $IsInTestMode = 0;
    my $LocaleId = 'en_US';
    my $CountryCode = 'DE';
    my $LineItemPaymentPayPalPlus = $self->_buildLineItemPaymentPayPalPlus($hParams, $IsInTestMode, $LocaleId, $CountryCode);

    my $wasSuccessful = $PaymentWallDataBuilder->extractData($LineItemPaymentPayPalPlus);
    my $hData = $PaymentWallDataBuilder->getData();

    is($wasSuccessful, 1, "$SubTestName - Test if call was successful");
    is($hData->{'approvalUrl'},'http://some.to/url', "$SubTestName - Test if URL is right");
    is($hData->{'mode'},'live', "$SubTestName - Test if mode is right");
    is($hData->{'language'},'en_US', "$SubTestName - Test if URL is right");
    is($hData->{'country'},'DE', "$SubTestName - Test if mode is right");
}

sub test_extractData_WrongURL {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder->new();

    my $hParams = {'ApprovalURL' => 'wrong_url'};
    my $IsInTestMode = 1;
    my $LineItemPaymentPayPalPlus = $self->_buildLineItemPaymentPayPalPlus($hParams, $IsInTestMode);

    my $wasSuccessful = $PaymentWallDataBuilder->extractData($LineItemPaymentPayPalPlus);
    my $hError = $PaymentWallDataBuilder->getError();

    is($wasSuccessful, 0, "$SubTestName - Test if call was not successful");
    is($hError->{'errorCode'},'FORMAT_NOT_URL', "$SubTestName - Test if Error is FORMAT_NOT_URL");
}

sub test_extractData_MissingIsInTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder->new();

    my $hParams = {'ApprovalURL' => 'http://some.to/url'};
    my $IsInTestMode = undef;
    my $LineItemPaymentPayPalPlus = $self->_buildLineItemPaymentPayPalPlus($hParams, $IsInTestMode);

    my $wasSuccessful = $PaymentWallDataBuilder->extractData($LineItemPaymentPayPalPlus);
    my $hError = $PaymentWallDataBuilder->getError();

    is($wasSuccessful, 0, "$SubTestName - Test if call was successful");
    is($hError->{'errorCode'},'PARAMETER_MISSING', "$SubTestName - Test if Error is PARAMETER_MISSING");
}

sub test_extractData_MissingLineItemPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder->new();

    my $wasSuccessful = $PaymentWallDataBuilder->extractData(undef);
    my $hError = $PaymentWallDataBuilder->getError();

    is($wasSuccessful, 0, "$SubTestName - Test if call was not successful");
    is($hError->{'errorCode'},'PARAMETER_MISSING', "$SubTestName - Test if Error is PARAMETER_MISSING");
}

sub _buildCountry {
    my $self = shift;
    my ($CountryCode) = @_;
    return {'Code2' => $CountryCode}
}

sub _buildAddress {
    my $self = shift;
    my ($CountryCode) = @_;
    my $hParams = {
        'Country' => $self->_buildCountry($CountryCode)
    };
    my $Address = DE_EPAGES::Test::Mock::EpagesObject->new(
    'DE_EPAGES::Address::API::Object::Address',
    $hParams);
    return $Address;
}

sub _buildBasket {
    my $self = shift;
    my ($CountryCode) = @_;
    my $hParams = {
        'ShippingAddress' => $self->_buildAddress($CountryCode)
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new(
    'DE_EPAGES::Order::API::Object::Basket',
    $hParams);
    return $Basket;
}

sub _buildLineItemContainer {
    my $self = shift;
    my ($LocaleID, $CountryCode) = @_;
    my $hParams = {
        'LocaleID' => $LocaleID,
        'BasketOrOrder' => $self->_buildBasket($CountryCode),
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new(
    'DE_EPAGES::Order::API::Object::LineItemContainer',
    $hParams);
    return $LineItemContainer;
}

sub _buildLineItemPaymentPayPalPlus {
    my $self = shift;
    my ($hParams, $IsInTestMode, $LocaleID, $CountryCode) = @_;
    $hParams->{'PaymentMethod'} = $self->_buildPaymentMethodPayPalPlus($IsInTestMode);
    $hParams->{'LineItemContainer'} = $self->_buildLineItemContainer($LocaleID, $CountryCode);
    my $LineItemPaymentPayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new(
    'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus',
    $hParams);
    return $LineItemPaymentPayPalPlus;
}

sub _buildPaymentMethodPayPalPlus {
    my $self = shift;
    my ($IsInTestMode) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus',
        $hAttributes
    );

    $Mockify->mock('isInTestMode', $IsInTestMode);
    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();