#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductSlideshowImageDeletionResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::Resource::ProductSlideshowImageDeletion
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductSlideshowImageDeletionResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::Resource::ProductSlideshowImageDeletion;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_deleteSlideshowImage_ProductNotVisible();
    $self->test_deleteSlideshowImage();

    done_testing();
}



sub test_deleteSlideshowImage_ProductNotVisible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsProductVisible = 0;
    my $Mockery = $self->_createMocks($IsProductVisible);
    my $Resource = $Mockery->get('SUT');

    my $Product = $Mockery->get('Product');
    my $aPathParams = ['testImage.png'];

    lives_ok( sub {
        $Resource->delete($Product, $aPathParams);
    }, "$SubTestName - test if delete can be called without errors");
}



sub test_deleteSlideshowImage {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsProductVisible = 1;
    my $Mockery = $self->_createMocks($IsProductVisible);
    my $Resource = $Mockery->get('SUT');

    my $Product = $Mockery->get('Product');
    my $aPathParams = ['testImage.png'];

    lives_ok( sub {
        $Resource->delete($Product, $aPathParams);
    }, "$SubTestName - test if delete can be called without errors");

    my $ProductImageService = $Mockery->get('ProductImageService');
    my $aArguments = GetParametersFromMockifyCall($ProductImageService, 'deleteSlideShowImage');
    is($aArguments->[0], 'testImage.png', "$SubTestName - test if deleteSlideShowImage was called with image name as argument");
}



sub _createMocks {
    my $self = shift;
    my ($IsProductVisible) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['ProductImageService'], sub {
        my ($ProductImageService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::Resource::ProductSlideshowImageDeletion', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createProductImageService', $ProductImageService, ['object']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductImageService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductImageService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('deleteSlideShowImage', undef, ['string']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Product', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'IsVisible' => $IsProductVisible,
            'GUID' => '124-456',
            'Site' => $Shop,
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();