#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductSlideshowResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::Resource::ProductSlideshow
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductSlideshowResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_postProductImage_ImageNameWithoutExtension();
    $self->test_postProductImage_ImageNameWithWrongExtension();
    $self->test_postProductImage_InvisibleProduct();
    $self->test_postProductImage_ProductWithTooManyImages();
    $self->test_postProductImage_ProductWithoutSlideshow();
    $self->test_postProductImage();
    $self->test_postProductImage_DuplicateImageName();

    done_testing();
}



sub test_postProductImage_ImageNameWithoutExtension {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('valid_product');

    throws_ok( sub {
        $ProductImageResource->post($Product, undef, 'testImage');
    }, qr/UNSUPPORTED_IMAGE_FORMAT/,
    "$SubTestName - test if error for unsupported image format was thrown.");
}



sub test_postProductImage_ImageNameWithWrongExtension {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('valid_product');

    throws_ok( sub {
        $ProductImageResource->post($Product, undef, 'testImage.exe');
    }, qr/UNSUPPORTED_IMAGE_FORMAT/,
    "$SubTestName - test if error for unsupported image format was thrown.");
}



sub test_postProductImage_InvisibleProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('invisible_product');
    my $Request = $Mockery->get('Request');

    lives_ok( sub {
        $ProductImageResource->post($Product, $Request, 'testImage.png');
    }, "$SubTestName - test if the post method can be called without errors");

    is(WasCalled($ProductImageResource, '_PerformSEOCheckAndUpdateObject') ,1,"$SubTestName - test if SEO update was performed");
    is($Product->{'updatePageCacheWasCalled'}, 1, "$SubTestName - test if page cache was updated");
}



sub test_postProductImage_ProductWithTooManyImages {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks({
        'ImageCount' => 200,
    });
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('valid_product');

    throws_ok( sub {
        $ProductImageResource->post($Product, undef, 'testImage.png');
    }, qr/PRODUCT_IMAGE_MAX_COUNT: PRODUCT_IMAGE_MAX_COUNT/,
    "$SubTestName - test if error for max image count was thrown.");
}



sub test_postProductImage_ProductWithoutSlideshow {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('product_without_slideshow');
    my $Request = $Mockery->get('Request');

    lives_ok( sub {
        $ProductImageResource->post($Product, $Request, 'testImage.png');
    }, "$SubTestName - test if the post method can be called without errors");

    is(WasCalled($ProductImageResource, '_PerformSEOCheckAndUpdateObject') ,1,"$SubTestName - test if SEO update was performed");
    is($Product->{'updatePageCacheWasCalled'}, 1, "$SubTestName - test if page cache was updated");
}



sub test_postProductImage {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('valid_product');
    my $Request = $Mockery->get('Request');

    lives_ok( sub {
        $ProductImageResource->post($Product, $Request, 'testImage.png');
    }, "$SubTestName - test if the post method can be called without errors");

    is(WasCalled($ProductImageResource, '_PerformSEOCheckAndUpdateObject') ,1,"$SubTestName - test if SEO update was performed");
    is($Product->{'updatePageCacheWasCalled'}, 1, "$SubTestName - test if page cache was updated");
}

sub test_postProductImage_DuplicateImageName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductImageResource = $Mockery->get('SUT');
    my $Product = $Mockery->get('valid_product');

    throws_ok( sub {
        $ProductImageResource->post($Product, undef, 'puttedTestImage.png');
    }, qr/ImageNameIsTaken/,
    "$SubTestName - test if error  ImageNameIsTaken is thrown.");

}


sub _createMocks {
    my $self = shift;
    my ($hParams) = @_;

    my $ImageCount = $hParams->{'ImageCount'} //= 3;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['ProductImageValidation', 'ProductImageService'], sub {
        my ($ProductImageValidation, $ProductImageService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::Resource::ProductSlideshow', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createProductImageValidation', $ProductImageValidation, ['object']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createProductImageService', $ProductImageService, ['object']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_PerformSEOCheckAndUpdateObject', undef, ['object']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductImageValidation', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductImageValidation', []);
        $Mockify->addMockWithReturnValue('getImageCount', $ImageCount);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductImageService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductImageService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('uploadAndScaleImage', undef, ['string', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('updateSlideShowString', undef, ['string']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Request', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('HTTP::Request', []);
        return $Mockify->getMockObject();
    });
    $Mockery->register('shop', [], sub {
        my $Product = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'ProtocolAndServerSSL' => 'http://www.myshop.test',
            'WebPath' => '/web/path',
        });
        return $Product;
    });
    $Mockery->register('valid_product', ['shop'], sub {
        my ($Shop) = @_;
        my $Product = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Site' => $Shop,
            'GUID' => '123-45-6',
            'IsVisible' => 1,
            'UseSlideshow' => 1,
            'ImagesSlideShow' => $self->_createImageSlideShow(),
            'WebPath' => '/web/path/myProduct',
            'PublicFiles' => [{
            'FileName' => 'puttedTestImage.png'
        }],
        });
        $Product->mock('updatePageCache', sub {
            my $MockedSelf = shift;
            $MockedSelf->{'updatePageCacheWasCalled'} = 1;
        });
        return $Product;
    });
    $Mockery->register('product_without_slideshow', ['shop'], sub {
        my ($Shop) = @_;
        my $Product = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Site' => $Shop,
            'GUID' => '123-45-6',
            'IsVisible' => 1,
            'UseSlideshow' => 0,
            'ImagesSlideShow' => $self->_createImageSlideShow(),
            'WebPath' => '/web/path/myProduct',
            'PublicFiles' => [],
        });
        $Product->mock('updatePageCache', sub {
            my $MockedSelf = shift;
            $MockedSelf->{'updatePageCacheWasCalled'} = 1;
        });
        return $Product;
    });
    $Mockery->register('invisible_product', ['shop'], sub {
        my ($Shop) = @_;
        my $Product = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Site' => $Shop,
            'GUID' => '123-45-6',
            'IsVisible' => 0,
            'UseSlideshow' => 1,
            'ImagesSlideShow' => $self->_createImageSlideShow(),
            'WebPath' => '/web/path/myProduct',
            'PublicFiles' => [{
            'FileName' => 'puttedTestImage.png'
        }],
        });
        $Product->mock('updatePageCache', sub {
            my $MockedSelf = shift;
            $MockedSelf->{'updatePageCacheWasCalled'} = 1;
        });
        return $Product;
    });

    return $Mockery->assemble();
}



sub _createImageSlideShow {
    my $self = shift;
    return [
        {
            'ImageName' => 'image123.jpg',
        },
        {
            'ImageName' => 'puttedTestImage.png',
        }
    ];
}

__PACKAGE__->runScript();