﻿#========================================================================================
# §package      DE_EPAGES::Shop::t::unit::Address
# §base         DE_EPAGES::Core::API::Test
# §state        public
#----------------------------------------------------------------------------------------
# §description  Testing whether a country is supported by a shop (e.g. in the checkout)
#========================================================================================
package DE_EPAGES::Shop::t::unit::Address;
use base DE_EPAGES::Core::API::Test;

use utf8;
use strict;
use Test::More;

use DE_EPAGES::Shop::API::Address qw ( CheckShopCountry );
use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;

use constant CYPRUS => 196;
use constant GERMANY => 276;
use constant SPAIN => 724;
use constant USA => 840;


sub test {
    my $self = shift;

    $self->test_CheckShopCountry();

    done_testing();
}

sub test_CheckShopCountry {
    my $self = shift;

    my $Shop = $self->_createTestShop();
    ok(!CheckShopCountry($Shop, CYPRUS),  "Cyprus is no shop country");
    ok( CheckShopCountry($Shop, GERMANY), "Germany is shop country"  );
    ok( CheckShopCountry($Shop, SPAIN),   "Spain is shop country"    );
    ok(!CheckShopCountry($Shop, USA),     "USA is no shop country"   );
    ok( CheckShopCountry($Shop, undef),   "undef is no country and therefore no invalid country" );

    return;
}


## Mocks

sub _createTestShop {
    my $self = shift;

    my $MockifyShop = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new()->buildShop({
            'ShopCountries' => [
                { 'CountryID' => GERMANY },
                { 'CountryID' => SPAIN }
            ]
        });

    return $MockifyShop->getMockObject();;
}

__PACKAGE__->runScript();

1;
