use strict;
use feature 'say';
use feature 'switch';

use Getopt::Long;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

sub Main {

    my $Provider;
    my $ShopRefAlias;
    my $Help;

    GetOptions(
        'help'       => \$Help,
        'provider=s' => \$Provider,
        'shopid=s'   => \$ShopRefAlias,
    );

    usage() if $Help;

    unless ($Provider) {
        print STDERR "Missing parameter -provider.\n\n";
        usage();
    }
    unless ($ShopRefAlias) {
        print STDERR "Missing parameter -shopid.\n\n";
        usage();
    }

    # find the shop in sitedb
    RunOnStore(
        Store => 'Site',
        Sub   => sub {
            my $ShopRef = LoadObjectByPath("/Providers/$Provider/ShopRefs/$ShopRefAlias");

            say 'Unity already registered' if ($ShopRef->get('IsUnityRegistered'));

            eval {
                $ShopRef->registerUnity;
            };
            if (ExistsError()) {
                my $Error = GetError();

                given ($Error->code) {
                    when (['UNITY_API_CALL_ERROR']) {
                        say 'Registering of the Unity shop failed. Please retry.';
                    }
                }

                Error($Error);
            }
            else {
                say 'Registering of the Unity shop was successful.';
            }
        }
    );

    eval { SynchronizeCache(); };
    Warning(GetError()) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -provider   provider alias
    -shopid     shop id (see PBO -> Shops)
flags:
    -help       show the command line options

Example:
    perl $0 -provider Distributor -shopid Store.DemoShop
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);

