#========================================================================================
# package      DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorize
# base         DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for PaymentMethodSpreedlyAuthorize
#========================================================================================
package DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorize;
use base DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly;

use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;

use constant PAYMENT_TYPE_AUTHORIZE => 'SpreedlyAuthorize';

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettingsSpreedlyAuthorize {
    my $self = shift;
    my ($Servlet,$PaymentMethod) = @_;

    my $URL = $self->_getCurrentURL($Servlet, $PaymentMethod);

    $Servlet->vars('StateURL', $URL);

    my $Shop = $PaymentMethod->getSite();
    my $IsInternalTestShop = $Shop->get('IsInternalTestShop');
    my $PaymentType = $PaymentMethod->get('PaymentType');
    my $ClientID = ($IsInternalTestShop) ? $PaymentType->get('TestAccountName') : $PaymentType->get('LiveAccountName');
    $Servlet->vars('ClientID', $ClientID);

    my $Form = $Servlet->form();
    my $ApiLoginID = $Form->value('APILoginID');
    my $TransactionKey = $Form->value('TransactionKey');

    if (isValid($ApiLoginID) && isValid($TransactionKey)) {
        $self->_createGateway($ApiLoginID, $TransactionKey, $Servlet);
    }

    $Servlet->vars('SiteUrl', $self->_getSiteUrl());

    return $Servlet;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    $Servlet->form->form($PaymentMethod, 'SpreedlyAuthorizeSettings');

    $self->SUPER::SaveSettings($Servlet);

}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _createGateway {
    my $self = shift;
    my ($ApiLoginID, $TransactionKey, $Servlet) = @_;

    my $hAdditionalBodyParams = {
        'api_login_id' => $ApiLoginID,
        'transaction_key' => $TransactionKey,
    };

    my $PaymentMethodSpreedly = DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly->new();
    $PaymentMethodSpreedly->SaveCredentials($Servlet, PAYMENT_TYPE_AUTHORIZE, $hAdditionalBodyParams);

    return;
}

sub _getCurrentURL {
    my $self = shift;
    my ($Servlet,$PaymentMethod) = @_;


    my $hFormValues = $Servlet->params();
    my $hParams = {
        'ViewAction' => $hFormValues->{'ViewAction'},
        'ObjectID'   => $PaymentMethod->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'UseSSL' => 1,
    };

    return BuildShopUrl($PaymentMethod, $hParams, $hOptions, $Servlet);
}

sub _getSiteUrl {
    my $self = shift;

    my $EpagesConf = DE_EPAGES::Core::API::IniConfig->new( FileName => $ENV{'EPAGES_CONFIG'}.'/epages.conf', CaseSensitive => 1 );

    return $EpagesConf->get( 'DE_EPAGES::Spreedly', 'SiteUrl');
}

1;
