package DE_EPAGES::Test::Mock::EpagesObjectMockFactory;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::Constants qw (
  ADDRESS
  BASKET
  CLASS
  CUSTOMER
  CUSTOMER_ATTRIBUTE
  CUSTOMER_ORDER
  LINE_ITEM_CONTAINER
  LINE_ITEM_COUPON
  LINE_ITEM_ECO_PARTICIPATION
  LINE_ITEM_PAYMENT
  LINE_ITEM_PAYMENT_DISCOUNT
  LINE_ITEM_PRODUCT
  LINE_ITEM_SHIPPING
  PAYMENT_TYPE
  PRODUCT
  SHIPPING_FULLFILLMENT_TYPE
  SHIPPING_OPTION
  SHIPPING_PROVIDER_TYPE
  SHOP
  SITE
  USER
);
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Test = PackageName->new;
#----------------------------------------------------------------------------------------
# description  Creates an instance of a Factory to build ePages object mocks.
#========================================================================================
sub new {
    my $class = shift;
    my $self = bless {'DefaultLanagueId' => 1}, $class;

    return $self;
}

#========================================================================================
# function     buildServlet
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildServlet( $Object );
#----------------------------------------------------------------------------------------
# description  Builds a mock for an epages servlet.
#----------------------------------------------------------------------------------------
# input        $Object | Epages object to set for servlet. | object
# return       Mockify instance for a servlet. | object
#========================================================================================
sub buildServlet {
    my $self = shift;
    my ($Object) = @_;
    return DE_EPAGES::Test::Mock::MockifyServlet->new($Object);
}

#========================================================================================
# function     build
# state        public
#----------------------------------------------------------------------------------------
# syntax       build( $PackageName, $hAttributes, $LanguageId );
#----------------------------------------------------------------------------------------
# description  builds a mock of the specified epages object with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $PackageName | package of the class to mock. | string
# input        $hAttributes (optional) | attributes to initalize for the mock. | refhash
# input        $LanguageId (optional) | id of the language to use. | integer
# return       mock of the specified epages object with the passed attributes. | object
#========================================================================================
sub build {
    my $self = shift;
    my ($PackageName, $hAttributes, $LanguageId) = @_;
    $LanguageId = $self->defaultLanguageId() unless defined $LanguageId;
    $hAttributes = {} unless defined $hAttributes;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $LanguageId);
}

#========================================================================================
# function     buildAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildAddress( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Address' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Address' with the passed attributes. | object
#========================================================================================
sub buildAddress {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(ADDRESS, $hAttributes);
}

#========================================================================================
# function     buildBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildBasket( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Basket' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Basket' with the passed attributes. | object
#========================================================================================
sub buildBasket {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(BASKET, $hAttributes);
}

#========================================================================================
# function     buildClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildClass( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Class' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Class' with the passed attributes. | object
#========================================================================================
sub buildClass {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(CLASS, $hAttributes);
}

#========================================================================================
# function     buildCustomer
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildCustomer( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Customer' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Customer' with the passed attributes. | object
#========================================================================================
sub buildCustomer {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(CUSTOMER, $hAttributes);
}

#========================================================================================
# function     buildCustomerAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildCustomerAttribute( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'CustomerAttribute' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'CustomerAttribute' with the passed attributes. | object
#========================================================================================
sub buildCustomerAttribute {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(CUSTOMER_ATTRIBUTE, $hAttributes);
}

#========================================================================================
# function     buildCustomerOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildCustomerOrder( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'CustomerOrder' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'CustomerOrder' with the passed attributes. | object
#========================================================================================
sub buildCustomerOrder {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(CUSTOMER_ORDER, $hAttributes);
}

#========================================================================================
# function     buildLineItemContainer
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemContainer( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemContainer' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemContainer' with the passed attributes. | object
#========================================================================================
sub buildLineItemContainer {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_CONTAINER, $hAttributes);
}

#========================================================================================
# function     buildLineItemCoupon
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemCoupon( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemCoupon' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemCoupon' with the passed attributes. | object
#========================================================================================
sub buildLineItemCoupon {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_COUPON, $hAttributes);
}

#========================================================================================
# function     buildLineItemEcoParticipation
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemEcoParticipation( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemEcoParticipation' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemEcoParticipation' with the passed attributes. | object
#========================================================================================
sub buildLineItemEcoParticipation {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_ECO_PARTICIPATION, $hAttributes);
}

#========================================================================================
# function     buildLineItemPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemPayment( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemPayment' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemPayment' with the passed attributes. | object
#========================================================================================
sub buildLineItemPayment {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_PAYMENT, $hAttributes);
}

#========================================================================================
# function     buildLineItemPaymentDiscount
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemPaymentDiscount( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemPaymentDiscount' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemPaymentDiscount' with the passed attributes. | object
#========================================================================================
sub buildLineItemPaymentDiscount {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_PAYMENT_DISCOUNT, $hAttributes);
}

#========================================================================================
# function     buildLineItemProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemProduct( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemProduct' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemProduct' with the passed attributes. | object
#========================================================================================
sub buildLineItemProduct {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_PRODUCT, $hAttributes);
}

#========================================================================================
# function     buildLineItemShipping
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildLineItemShipping( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'LineItemShipping' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'LineItemShipping' with the passed attributes. | object
#========================================================================================
sub buildLineItemShipping {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(LINE_ITEM_SHIPPING, $hAttributes);
}

#========================================================================================
# function     buildPaymentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildPaymentType( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'PaymentType' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'PaymentType' with the passed attributes. | object
#========================================================================================
sub buildPaymentType {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(PAYMENT_TYPE, $hAttributes);
}

#========================================================================================
# function     buildProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildProduct( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Product' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Product' with the passed attributes. | object
#========================================================================================
sub buildProduct {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(PRODUCT, $hAttributes);
}

#========================================================================================
# function     buildShippingFulfillmentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildShippingFulfillmentType( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'ShippingFulfillmentType' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'ShippingFulfillmentType' with the passed attributes. | object
#========================================================================================
sub buildShippingFulfillmentType {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(SHIPPING_FULLFILLMENT_TYPE, $hAttributes);
}

#========================================================================================
# function     buildShippingOption
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildShippingOption( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'ShippingOption' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'ShippingOption' with the passed attributes. | object
#========================================================================================
sub buildShippingOption {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(SHIPPING_OPTION, $hAttributes);
}

#========================================================================================
# function     buildShippingProviderType
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildShippingProviderType( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'ShippingProviderType' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'ShippingProviderType' with the passed attributes. | object
#========================================================================================
sub buildShippingProviderType {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(SHIPPING_PROVIDER_TYPE, $hAttributes);
}

#========================================================================================
# function     buildShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildShop( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Shop' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Shop' with the passed attributes. | object
#========================================================================================
sub buildShop {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(SHOP, $hAttributes);
}

#========================================================================================
# function     buildSite
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildSite( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'Site' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'Site' with the passed attributes. | object
#========================================================================================
sub buildSite {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(SITE, $hAttributes);
}

#========================================================================================
# function     buildUser
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildUser( $hAttributes );
#----------------------------------------------------------------------------------------
# description  builds a mock of the epages object 'User' with the passed attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributes | attributes to initalize for the mock. | refhash
# return       mock of the epages object 'User' with the passed attributes. | object
#========================================================================================
sub buildUser {
    my $self = shift;
    my ($hAttributes) = @_;

    return $self->build(USER, $hAttributes);
}

#========================================================================================
# function     defaultLanguageId
# state        public
#----------------------------------------------------------------------------------------
# syntax       defaultLanguageId( $DefaultLanguageId );
#----------------------------------------------------------------------------------------
# description  get-setter for the used default language id
#----------------------------------------------------------------------------------------
# input        $DefaultLanguageId | default language id to use | integer
# return       default language id | integer
#========================================================================================
sub defaultLanguageId {
    my $self = shift;
    my ($DefaultLanguageId) = @_;
    return $self->{'DefaultLanagueId'} unless defined $DefaultLanguageId;
    $self->{'DefaultLanagueId'} = $DefaultLanguageId;
}

1;