#========================================================================================
# package      DE_EPAGES::Test::Mock::MockifyArgumentMatchers
# state        public
#----------------------------------------------------------------------------------------
# description  Wrapper functions for Mockify argument matchers
#========================================================================================
package DE_EPAGES::Test::Mock::MockifyArgumentMatchers;
use base qw ( Exporter );

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw (
    TestString
    TestInteger
    TestArray
    TestHash
);

use strict;



our @EXPORT_OK = qw (
    String
    Int
    ArrayRef
    HashRef
    Object
    Any
    Undef
);



#========================================================================================
# function     String
# state        public
#----------------------------------------------------------------------------------------
# syntax       String();
# syntax       String( $Expected );
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for string arguments.
#----------------------------------------------------------------------------------------
# input        $Expected | An expected string argument (optional) | string
# return       A Mockify argument matcher | refhash
#========================================================================================
sub String {
    my ($Arg) = @_;
    if (defined $Arg) {
        TestString('String argument', $Arg);
        {'string' => $Arg};
    } else {
        'string';
    }
}



#========================================================================================
# function     Int
# state        public
#----------------------------------------------------------------------------------------
# syntax       Int();
# syntax       Int( $Expected );
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for integer arguments.
#----------------------------------------------------------------------------------------
# input        $Expected | An expected integer argument (optional) | integer
# return       A Mockify argument matcher | refhash
#========================================================================================
sub Int {
    my ($Arg) = @_;
    if (defined $Arg) {
        TestInteger('Integer argument', $Arg);
        {'int' => $Arg};
    } else {
        'int';
    }
}



#========================================================================================
# function     ArrayRef
# state        public
#----------------------------------------------------------------------------------------
# syntax       ArrayRef();
# syntax       ArrayRef( $Expected );
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for array reference arguments.
#----------------------------------------------------------------------------------------
# input        $Expected | An expected array reference argument (optional) | refarray
# return       A Mockify argument matcher | refhash
#========================================================================================
sub ArrayRef {
    my ($Arg) = @_;
    if (defined $Arg) {
        TestArray('Array reference argument', $Arg);
        {'arrayref' => $Arg};
    } else {
        'arrayref';
    }
}



#========================================================================================
# function     HashRef
# state        public
#----------------------------------------------------------------------------------------
# syntax       HashRef();
# syntax       HashRef( $Expected );
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for hash reference arguments.
#----------------------------------------------------------------------------------------
# input        $Expected | An expected hash reference argument (optional) | refhash
# return       A Mockify argument matcher | refhash
#========================================================================================
sub HashRef {
    my ($Arg) = @_;
    if (defined $Arg) {
        TestHash('Hash reference argument', $Arg);
        {'hashref' => $Arg};
    } else {
        'hashref';
    }
}



#========================================================================================
# function     Object
# state        public
#----------------------------------------------------------------------------------------
# syntax       Object();
# syntax       Object( $Expected );
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for object arguments.
#----------------------------------------------------------------------------------------
# input        $Expected | An expected package name (optional) | string
# return       A Mockify argument matcher | refhash
#========================================================================================
sub Object {
    my ($Arg) = @_;
    if (defined $Arg) {
        TestString('Package name argument', $Arg);
        {'object' => $Arg};
    } else {
        'object';
    }
}



#========================================================================================
# function     Any
# state        public
#----------------------------------------------------------------------------------------
# syntax       Any();
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for any arguments.
#----------------------------------------------------------------------------------------
# return       A Mockify argument matcher | refhash
#========================================================================================
sub Any {
    Error('Argument matcher Any does not take any arguments!') if (scalar @_ > 0);
    'any';
}



#========================================================================================
# function     Undef
# state        public
#----------------------------------------------------------------------------------------
# syntax       Undef();
#----------------------------------------------------------------------------------------
# description  Creates a Mockify argument matcher for undefined arguments.
#----------------------------------------------------------------------------------------
# return       A Mockify argument matcher | refhash
#========================================================================================
sub Undef {
    Error('Argument matcher Undef does not take any arguments!') if (scalar @_ > 0);
    'undef';
}

1;
