#========================================================================================
# package      DE_EPAGES::Test::t::unit::MockifyArgumentMatchers
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Test::Mock::MockifyArgumentMatchers
#========================================================================================
package DE_EPAGES::Test::t::unit::Mockify;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyArgumentMatchers qw ( String Int ArrayRef HashRef Object Any Undef );
use Test::More;
use Test::Exception;

use strict;



sub test {
    my $self = shift;

    $self->test_String_anyString();
    $self->test_String_specificString();
    $self->test_String_wrongArgument();

    $self->test_Int_anyInt();
    $self->test_Int_specificInt();
    $self->test_Int_wrongArgument();

    $self->test_ArrayRef_anyArray();
    $self->test_ArrayRef_specificArray();
    $self->test_ArrayRef_wrongArgument();

    $self->test_HashRef_anyHash();
    $self->test_HashRef_specificHash();
    $self->test_HashRef_wrongArgument();

    $self->test_Object_anyObject();
    $self->test_Object_specificObject();
    $self->test_Object_wrongArgument();

    $self->test_Any();
    $self->test_Any_wrongArgument();

    $self->test_Undef();
    $self->test_Undef_wrongArgument();

    done_testing();
}



sub test_String_anyString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = String();
    is($ArgumentMatcher, 'string', "$SubTestName - test if correct matcher for any string was returned.");
}



sub test_String_specificString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = String('test string');
    is_deeply($ArgumentMatcher, {'string'=>'test string'}, "$SubTestName - test if correct matcher for specific string was returned.");
}



sub test_String_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    throws_ok(
        sub { String([]) },
        qr/String argument is an object/,
        "$SubTestName - test if matcher throws an error if called with argument which is not a string."
    );
}



sub test_Int_anyInt {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Int();
    is($ArgumentMatcher, 'int', "$SubTestName - test if correct matcher for any integer was returned.");
}



sub test_Int_specificInt {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Int(12345);
    is_deeply($ArgumentMatcher, {'int'=>12345}, "$SubTestName - test if correct matcher for specific integer was returned.");
}



sub test_Int_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    throws_ok(
        sub { Int('string') },
        qr/Integer argument is no integer number/,
        "$SubTestName - test if matcher throws an error if called with argument which is not an integer."
    );
}



sub test_ArrayRef_anyArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = ArrayRef();
    is($ArgumentMatcher, 'arrayref', "$SubTestName - test if correct matcher for any array was returned.");
}



sub test_ArrayRef_specificArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = ArrayRef([123, 'abc', 'test']);
    is_deeply($ArgumentMatcher, {'arrayref'=>[123, 'abc', 'test']}, "$SubTestName - test if correct matcher for specific array was returned.");
}


sub test_ArrayRef_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    throws_ok(
        sub { ArrayRef(123) },
        qr/Array reference argument is no array reference/,
        "$SubTestName - test if matcher throws an error if called with argument which is not an array reference."
    );
}



sub test_HashRef_anyHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = HashRef();
    is($ArgumentMatcher, 'hashref', "$SubTestName - test if correct matcher for any hash was returned.");
}



sub test_HashRef_specificHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = HashRef({'abc'=>123, 'test'=>{'some'=>'value'}});
    is_deeply($ArgumentMatcher, {'hashref'=>{'abc'=>123, 'test'=>{'some'=>'value'}}}, "$SubTestName - test if correct matcher for specific hash was returned.");
}



sub test_HashRef_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    throws_ok(
        sub { HashRef(123) },
        qr/Hash reference argument is no hash reference/,
        "$SubTestName - test if matcher throws an error if called with argument which is not an hash reference."
    );
}



sub test_Object_anyObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Object();
    is($ArgumentMatcher, 'object', "$SubTestName - test if correct matcher for any object was returned.");
}



sub test_Object_specificObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Object('DE_EPAGES::Shop::API::Object::Shop');
    is_deeply($ArgumentMatcher, {'object'=>'DE_EPAGES::Shop::API::Object::Shop'}, "$SubTestName - test if correct matcher for specific object was returned.");
}



sub test_Object_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    throws_ok(
        sub { Object([]) },
        qr/Package name argument is an object/,
        "$SubTestName - test if matcher throws an error if called with argument which is not an package name."
    );
}



sub test_Any {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Any();
    is($ArgumentMatcher, 'any', "$SubTestName - test if correct matcher for any argument was returned.");
}



sub test_Any_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];


    throws_ok(
        sub { Any('some argument') },
        qr/Argument matcher Any does not take any arguments!/,
        "$SubTestName - test if matcher throws an error if an argument is provided when no argument is expected."
    );
}



sub test_Undef {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ArgumentMatcher = Undef();
    is($ArgumentMatcher, 'undef', "$SubTestName - test if correct matcher for undef argument was returned.");
}



sub test_Undef_wrongArgument {
    my $self = shift;
    my $SubTestName = (caller(0))[3];


    throws_ok(
        sub { Undef('some argument') },
        qr/Argument matcher Undef does not take any arguments!/,
        "$SubTestName - test if matcher throws an error if an argument is provided when no argument is expected."
    );
}

__PACKAGE__->runScript();
